/*

  This file is a part of GRALE, a library to facilitate the simulation
  and inversion of gravitational lenses.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

/**
 * \file real2dlensplanefunction.h
 */

#ifndef GRALE_REAL2DLENSPLANEFUNCTION_H

#define GRALE_REAL2DLENSPLANEFUNCTION_H

#include "graleconfig.h"
#include "real2dfunction.h"
#include "lensplane.h"

namespace grale
{

/** This class implements a 2D function which can be used to evaluate approximately the mass density
 *  corresponding to the deflection field that was calculated.
 */
class GRALE_IMPORTEXPORT Real2DLensPlaneFunction : public Real2DFunction
{
public:	
	/** Initializes the function.
	 *  Initializes the function. Care must me taken that the lensplane \c lensPlane is valid
	 *  as long as the function is used.
	 *  \param lensPlane The lensplane for which the mass density should be calculated.
	 */
	Real2DLensPlaneFunction(LensPlane &lensPlane);

	~Real2DLensPlaneFunction();
	double operator()(Vector2D<double> theta) const;
private:
	LensPlane &m_lensPlane;
	double m_densFactor;
};

} // end namespace

#endif // GRALE_REAL2DLENSPLANEFUNCTION_H

