/*

  This file is a part of GRALE, a library to facilitate the simulation
  and inversion of gravitational lenses.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#include "graleconfig.h"
#include "doublepeakdistribution.h"
#include "realtype.h"
#include <stdint.h>
#include <time.h>

#include "debugnew.h"

namespace grale
{

DoublePeakDistribution::DoublePeakDistribution(double width, const mogal::RandomNumberGenerator *pRndGen) : ProbabilityDistribution(pRndGen)
{
	m_width = width;
	m_z0 = 0.25*(1.0+EXP(-width));
	m_z1 = 0.25*(3.0-EXP(-width));
}

DoublePeakDistribution::~DoublePeakDistribution()
{
}

double DoublePeakDistribution::pickNumber() const
{
	double z, y;

	z = pickRandomNumber();

	if (z < m_z0)
	{
		y = (1.0/m_width)*LN((4.0*z)/((1.0+EXP(-m_width))));
	}
	else if (z > m_z1)
	{
		y = (1.0/m_width)*LN((1.0+EXP(m_width))/(4.0*(1.0-z)));
	}
	else
	{
		y = (1.0/m_width)*LN((2.0*z-1.0)+SQRT((2.0*z-1.0)*(2.0*z-1.0)+EXP(-m_width)))+1.0;
	}

	return y;
}

} // end namespace

