/*

  This file is a part of GRALE, a library to facilitate the simulation
  and inversion of gravitational lenses.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#ifndef OPENCLKERNEL_H

#define OPENCLKERNEL_H

#include "graleconfig.h"

#ifdef GRALECONFIG_SUPPORT_OPENCL

#include <errut/errorbase.h>
#ifdef GRALECONFIG_OPENCL_OPENCL_CL_H
	#include <OpenCL/cl.h>
#else
	#include <CL/cl.h>
#endif // GRALECONFIG_OPENCL_OPENCL_CL_H
#include <string>

namespace grale
{

class GRALE_IMPORTEXPORT OpenCLKernel : public errut::ErrorBase
{
public:
	OpenCLKernel();
	~OpenCLKernel();

	bool init();
	bool loadKernel(const std::string &program, const std::string &kernelName, std::string &failLog);
	bool destroy();

	cl_context getContext()										{ return m_context; }
	cl_kernel getKernel() 										{ return m_kernel; }
	cl_command_queue getCommandQueue()		 						{ return m_queue; }
private:
	static std::string getCLErrorString(int errNum);
	void releaseAll();

	bool m_init;
	std::string m_currentProgram, m_currentKernel;
	
	cl_context m_context;
	cl_command_queue m_queue;
	cl_program m_program;
	cl_kernel m_kernel;
	cl_device_id *m_pDevices;
	int m_deviceIndex;
};

} // end namespace

#endif // GRALECONFIG_SUPPORT_OPENCL

#endif // OPENCLKERNEL_H
