/*

  This file is a part of GRALE, a library to facilitate the simulation
  and inversion of gravitational lenses.

  Copyright (C) 2008-2012 Jori Liesenborgs

  Contact: jori.liesenborgs@gmail.com
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
  
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  
*/

#ifndef GRALE_IMAGEPLANEPNGWRITER_H

#define GRALE_IMAGEPLANEPNGWRITER_H

#include "graleconfig.h"

#ifdef GRALECONFIG_SUPPORT_QT5

#include "imageplanewriter.h"
#include "realtype.h"
#include <string>

namespace grale
{

class ImagePlane;

class GRALE_IMPORTEXPORT ImagePlanePNGWriter : public ImagePlaneWriter
{
public:
	ImagePlanePNGWriter(const std::string &fname,bool images,bool sources,bool criticallines,bool caustics)	{ im = images; srcs = sources; crit = criticallines; caust = caustics; filename = fname; magnify = true; sourcergb = true; sourceR = 0; sourceG = 32768; sourceB = 0; linewidth = 3; critR = 65535; critG = 0; critB = 0; caustR = 0; caustG = 0; caustB = 65535; }
	void setMagnify(bool m) 										{ magnify = m; }
	void setSourceRGB(int r,int g,int b)									{ sourceR = r; sourceG = g; sourceB = b; sourcergb = true; }
	void setSourceDim(double d = 2.0L)									{ sourcedim = d; sourcergb = false; }
	bool write(ImagePlane *p, int subSamples);
	void setLineWidth(int lw)										{ linewidth = lw; }
	void setCriticalLineRGB(int r,int g,int b)								{ critR = r; critG = g; critB = b; }
	void setCausticRGB(int r,int g,int b)									{ caustR = r; caustG = g; caustB = b; }
private:
	std::string filename;
	bool im,srcs,crit,caust;
	bool magnify;
	bool sourcergb;
	double sourcedim;
	int sourceR,sourceG,sourceB;
	int critR,critG,critB;
	int caustR,caustG,caustB;
	int linewidth;
};

} // end namespace

#endif // GRALECONFIG_SUPPORT_QT5

#endif // GRALE_IMAGEPLANEPNGWRITER_H

